@@ -29,8 +29,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <libedataserver/e-memory.h>
-
 #include "camel-file-utils.h"
 #include "camel-private.h"
 #include "camel-store.h"
@@ -141,7 +139,7 @@ camel_imap_store_summary_new (void)
 /**
  * camel_imap_store_summary_full_name:
  * @s: 
- * @path: 
+ * @full_name: 
  * 
  * Retrieve a summary item by full name.
  *
